clist <- c("de", "fr", "cz", "ie", "se")

pal <- RColorBrewer::brewer.pal(3, "Set1")

plot.val <- function(x, lwd=1, pch=1, cex=.8, col="black", lty="solid") {
    y <- as.numeric(substr(names(x), 3, 4))
    y <- ifelse(y < 70, 2000 + y, 1900 + y)
    points(y, x, pch=pch, cex=cex, col=col)
    lines(1970:2020, predict(loess(x ~ y, span=1), 1970:2020), lwd=lwd, col=col, lty=lty)
}


# Figure 1
layout(matrix(1:21, 7, 3, byrow=TRUE), heights=c(1, rep(5, 5), 3))
par(oma=c(0, 1.5, 0, 0.1))
par(mar=c(0, 2, 0, 0))
plot.new()
title("Employment rate (%)", line=-1)

plot.new()
title("Employment-earnings correlation", line=-1, xpd=NA)

plot.new()
title("Earnings correlation", line=-1)

for(c in clist) {
    par(mar=c(1, 2, 1, 0))

    # Share with nonzero earnings
    tab <- tabs[,, substr(dimnames(tabs)[[3]], 1, 2) == c]
    names(dimnames(tab)) <- c("M", "F", "S")

    share0F <- prop.table(margin.table(tab, c(2, 3)), 2)["Inactive",]*100
    share0M <- prop.table(margin.table(tab, c(1, 3)), 2)["Inactive",]*100

    plot(NULL, xaxt="n", yaxt="n", xlim=c(1973, 2016), ylim=c(25, 100),
         yaxs="i", xlab="", ylab="", main="")
    mtext(countrynames[toupper(c)], 2, 2, font=2, cex=0.9, xpd=NA)
    abline(h=seq(0, 100, by=25), lty="dotted", col="light grey"); box()
    axis(1, cex.axis=0.9, mgp=c(0, .5, 0), tcl=-.4, labels=c == tail(clist, 1))
    #axis(2, cex.axis=0.9, mgp=c(0, .5, 0), tcl=-.4, labels=F)
    axis(2, cex.axis=0.9, mgp=c(0, .5, 0), tcl=-.4, yaxp=c(0, 100, 4))
    # mtext("Proportion (%)", 2, 1.5, cex=0.7)
    plot.val(100-share0F, pch=17, col="#d95f02", lwd=1.5)
    plot.val(100-share0M, pch=16, col="#7570b3", lwd=1.5, lty="dashed")


    # Employment-earnings correlation
    pos <- which(grepl("correlation between her earnings and his employment", lines))[-1]
    corrpempf <- as.numeric(sapply(strsplit(lines[pos+5], " +"), function(x) x[4]))
    names(corrpempf) <- substr(lines[pos], 1, 4)
    corrpempf <- corrpempf[codes]
    
    pos <- which(grepl("correlation between his earnings and her employment", lines))[-1]
    corrpempm <- as.numeric(sapply(strsplit(lines[pos+5], " +"), function(x) x[4]))
    names(corrpempm) <- substr(lines[pos], 1, 4)
    corrpempm <- corrpempm[codes]

    plot(NULL, xaxt="n", yaxt="n", xlim=c(1973, 2016), ylim=c(-0.15, 0.18),
         xlab="", ylab="", main="")
    abline(h=seq(-1, 1, by=0.1), lty="dotted", col="light grey"); box()
    abline(h=0, col="grey"); box()
    axis(1, cex.axis=0.9, mgp=c(0, .5, 0), tcl=-.4, labels=c == tail(clist, 1))
    #axis(2, tcl=-.4, labels=F)
    axis(2, cex.axis=0.9, mgp=c(0, .5, 0), tcl=-.4, at=seq(-1, 1, by=0.1))
    # mtext("Correlation", 2, 1.5, cex=0.7)
    plot.val(corrpempm[substr(names(corrpempm), 1, 2) == c], pch=17, col="#d95f02", lwd=1.5)
    plot.val(corrpempf[substr(names(corrpempf), 1, 2) == c], pch=16, col="#7570b3", lwd=1.5, lty="dashed")


    # Correlation between partners' earnings
    pos <- which(grepl("correlation between his and her earnings, including zero-earners", lines))[-1]
    corr <- as.numeric(sapply(strsplit(lines[pos+5], " +"), function(x) x[4]))
    names(corr) <- substr(lines[pos], 1, 4)
    corr <- corr[codes]
    
    pos <- which(grepl("correlation between his and her earnings among dual earners", lines))[-1]
    corrd <- as.numeric(sapply(strsplit(lines[pos+5], " +"), function(x) x[4]))
    names(corrd) <- substr(lines[pos], 1, 4)
    corrd <- corrd[codes]

    plot(NULL, xaxt="n", yaxt="n", xlim=c(1973, 2016), ylim=c(-0.1, 0.4),
         xlab="", ylab="", main="")
    grid(NA, NULL)
    abline(h=0, col="grey"); box()
    axis(1, cex.axis=0.9, mgp=c(0, .5, 0), tcl=-.4, labels=c == tail(clist, 1))
    axis(2, labels=FALSE, tcl=-.4)
    axis(2, cex.axis=0.9, mgp=c(0, .5, 0), tcl=-.4, at=seq(-1, 1, by=.2))
    # mtext("Correlation", 2, 1.5, cex=0.7)
    plot.val(corr[substr(names(corr), 1, 2) == c], pch=16, col="#d95f02", lwd=1.5)
    plot.val(corrd[substr(names(corrd), 1, 2) == c], pch=17, col="#7570b3", lwd=1.5, lty="dashed")
}
par(mar=c(0, 0, 0, 0))
plot.new()
legend("center",
       c("Man's employment", "Woman's employment"), bty="n",
       col=c("#7570b3", "#d95f02"), lwd=1.5, pch=16:17,
       lty=c("dashed", "solid"), cex=1.2, seg.len=3)

plot.new()
legend("center", c("Man's employment", "Woman's employment"),
       col=c("#7570b3", "#d95f02"), lwd=1.5, pch=16:17, bty="n",
       lty=c("dashed", "solid"), cex=1.2, seg.len=3)

plot.new()
legend("center",
       c("Dual-earner couples", "All couples"), bty="n",
       col=c("#7570b3", "#d95f02"), lwd=1.5, pch=16:17,
       lty=c("dashed", "solid"), cex=1.2, seg.len=3)


# Figure A5
pal <- RColorBrewer::brewer.pal(3, "Set1")

cairo_pdf("Descriptive statistics all countries.pdf", height=16)
layout(matrix(1:((21+2)*3), 21+2, 3, byrow=TRUE), heights=c(1, rep(5, 21+2), 3))
par(oma=c(0, 1.5, 0, 0))
par(mar=c(0, 2, 0, 0))
plot.new()
title("Employment rate (%)", line=-1)

plot.new()
title("Employment-earnings correlation", line=-1, xpd=NA)

plot.new()
title("Earnings correlation", line=-1)

for(c in countries) {
    par(mar=c(0.5, 2, 0.5, 0.1))
    
    # Share with nonzero earnings
    tab <- tabs[,, substr(dimnames(tabs)[[3]], 1, 2) == c]
    names(dimnames(tab)) <- c("M", "F", "S")
    
    share0F <- prop.table(margin.table(tab, c(2, 3)), 2)["Inactive",]*100
    share0M <- prop.table(margin.table(tab, c(1, 3)), 2)["Inactive",]*100
    
    plot(NULL, xaxt="n", yaxt="n", xlim=c(1973, 2016), ylim=c(25, 100),
         yaxs="i", xlab="", ylab="", main="")
    mtext(countrynames[toupper(c)], 2, 2, font=2, cex=0.6, xpd=NA)
    abline(h=seq(0, 100, by=25), lty="dotted", col="light grey"); box()
    axis(1, cex.axis=0.7, mgp=c(0, .5, 0), tcl=-.4, labels=c == tail(countries, 1))
    #axis(2, cex.axis=0.9, mgp=c(0, .5, 0), tcl=-.4, labels=F)
    axis(2, cex.axis=0.7, mgp=c(0, .5, 0), tcl=-.4, yaxp=c(0, 100, 4))
    # mtext("Proportion (%)", 2, 1.5, cex=0.7)
    plot.val(100-share0F, pch=17, col="#d95f02", lwd=1.5)
    plot.val(100-share0M, pch=16, col="#7570b3", lwd=1.5, lty="dashed")
    
    
    # Employment-earnings correlation
    pos <- which(grepl("correlation between her earnings and his employment", lines))[-1]
    corrpempf <- as.numeric(sapply(strsplit(lines[pos+5], " +"), function(x) x[4]))
    names(corrpempf) <- substr(lines[pos], 1, 4)
    corrpempf <- corrpempf[codes]
    
    pos <- which(grepl("correlation between his earnings and her employment", lines))[-1]
    corrpempm <- as.numeric(sapply(strsplit(lines[pos+5], " +"), function(x) x[4]))
    names(corrpempm) <- substr(lines[pos], 1, 4)
    corrpempm <- corrpempm[codes]
    
    plot(NULL, xaxt="n", yaxt="n", xlim=c(1973, 2016), ylim=c(-0.17, 0.20),
         yaxs="i", xlab="", ylab="", main="")
    abline(h=seq(-1, 1, by=0.1), lty="dotted", col="light grey")
    abline(h=0, col="grey"); box()
    axis(1, cex.axis=0.7, mgp=c(0, .5, 0), tcl=-.4, labels=c == tail(countries, 1))
    #axis(2, tcl=-.4, labels=F)
    axis(2, cex.axis=0.7, mgp=c(0, .5, 0), tcl=-.4, at=seq(-1, 1, by=0.1))
    # mtext("Correlation", 2, 1.5, cex=0.7)
    plot.val(corrpempm[substr(names(corrpempm), 1, 2) == c], pch=17, col="#d95f02", lwd=1.5)
    plot.val(corrpempf[substr(names(corrpempf), 1, 2) == c], pch=16, col="#7570b3", lwd=1.5, lty="dashed")
    
    
    # Correlation between partners' earnings
    pos <- which(grepl("correlation between his and her earnings, including zero-earners", lines))[-1]
    corr <- as.numeric(sapply(strsplit(lines[pos+5], " +"), function(x) x[4]))
    names(corr) <- substr(lines[pos], 1, 4)
    corr <- corr[codes]
    
    pos <- which(grepl("correlation between his and her earnings among dual earners", lines))[-1]
    corrd <- as.numeric(sapply(strsplit(lines[pos+5], " +"), function(x) x[4]))
    names(corrd) <- substr(lines[pos], 1, 4)
    corrd <- corrd[codes]
    
    plot(NULL, xaxt="n", yaxt="n", xlim=c(1973, 2016), ylim=c(-0.17, 0.5),
         xlab="", ylab="", main="")
    abline(h=seq(-1, 1, by=0.2), lty="dotted", col="light grey")
    abline(h=0, col="grey"); box()
    axis(1, cex.axis=0.7, mgp=c(0, .5, 0), tcl=-.4, labels=c == tail(countries, 1))
    #axis(2, labels=FALSE, tcl=-.4)
    axis(2, cex.axis=0.7, mgp=c(0, .5, 0), tcl=-.4, at=seq(-1, 1, by=.2))
    # mtext("Correlation", 2, 1.5, cex=0.7)
    plot.val(corr[substr(names(corr), 1, 2) == c], pch=16, col="#d95f02", lwd=1.5)
    plot.val(corrd[substr(names(corrd), 1, 2) == c], pch=17, col="#7570b3", lwd=1.5, lty="dashed")
}
par(mar=c(0, 0, 0, 0))
plot.new()
legend("center",
       c("Man's employment", "Woman's employment"), bty="n",
       col=c("#7570b3", "#d95f02"), lwd=1.5, pch=16:17,
       lty=c("dashed", "solid"), cex=1, seg.len=3)

plot.new()
legend("center", c("Man's employment", "Woman's employment"),
       col=c("#7570b3", "#d95f02"), lwd=1.5, pch=16:17, bty="n",
       lty=c("dashed", "solid"), cex=1, seg.len=3)

plot.new()
legend("center",
       c("Dual-earner couples", "All couples"), bty="n",
       col=c("#7570b3", "#d95f02"), lwd=1.5, pch=16:17,
       lty=c("dashed", "solid"), cex=1, seg.len=3)
dev.off()



# Scenarios
cov.w <- function(y, w, y.mediansM=y, y.mediansF=y) {
    m <- medians[as.character(tab$M[tab$S==y]), "M", as.character(y.mediansM)]
    f <- medians[as.character(tab$F[tab$S==y]), "F", as.character(y.mediansF)]
    tot <- m + f
    w <- w[tab$S==y]
    tot.mean <- sum(tot * w)/sum(w)
    m.mean <- sum(m * w)/sum(w)
    f.mean <- sum(f * w)/sum(w)
    
    res <- cov.wt(cbind(Total=tot, M=m, F=f),
                  w, method="ML")
    cov <- c(res$cov)
    names(cov) <- outer(c("Total", "M", "F"), c("Total", "M", "F"), FUN=paste, sep=":")
    c(cov, "Total"=tot.mean, "M"=m.mean, "F"=f.mean)
}

load("models.RData")

# Figure 2
par(mfrow=c(3, 2), mar=c(1.5, 3, 1.5, 1), cex=0.9)

pal <- RColorBrewer::brewer.pal(3, "Set1")

for(c in clist) {
    mods <- models[[c]]
    mnull <- mods$null
    mstable <- mods$stable
    mMF0 <- mods$MF0
    mMF <- mods$MF
    mMFT <- mods$MFT
    
    tab <- mnull$data

    years <- as.character(unique(tab$S))
    covnull <- sapply(years, cov.w, fitted(mnull))
    covstable <- sapply(years, cov.w, fitted(mstable))
    covMF0 <- sapply(years, cov.w, fitted(mMF0))
    covMF <- sapply(years, cov.w, fitted(mMF))
    covMFT <- sapply(years, cov.w, fitted(mMFT))
    covsat <- sapply(years, cov.w, tab$Freq)
    
    cvnull <- sqrt(covnull["Total:Total",])/covnull["Total",]
    cvstable <- sqrt(covstable["Total:Total",])/covstable["Total",]
    cvMF0 <- sqrt(covMF0["Total:Total",])/covMF0["Total",]
    cvMF <- sqrt(covMF["Total:Total",])/covMF["Total",]
    cvMFT <- sqrt(covMFT["Total:Total",])/covMFT["Total",]
    cvsat <- sqrt(covsat["Total:Total",])/covsat["Total",]
    
    plot(NULL, xlim=c(1973, 2016), ylim=c(.35, .8), type="b", main=countrynames[toupper(c)],
         xlab="", ylab="", xaxt="n")
    axis(1, mgp=c(0, .5, 0), tcl=-.4, labels=c == tail(clist, 1) | c == clist[4])
    if(c == clist[1] || c == clist[3] || c == clist[5])
        title(ylab="Coefficient of variation", line=2)
    grid(NA, NULL)
    #plot.val(cvnull, lwd=3, pch=1, col="grey")
    plot.val(cvstable, lwd=3, pch=4, col=pal[3])
    #plot.val(cvMF0, lwd=3, pch=19, col=pal[2])
    plot.val(cvMF, lwd=3, pch=18, col=pal[2])
    plot.val(cvMFT, lwd=3, pch=17, col=pal[1])
    plot.val(cvsat, lwd=3, pch=15)
}
plot.new()
par(mar=c(0, 0, 0, 0))
legend("center", c("S1: only earnings distributions change",
                   "S2: + change in employment rates",
                   "S3: + change in employment-partner's\nearnings association",
                   "S4: + change in association among\ndual-earner couples"),
       title=expression(bold("Counterfactual scenarios:")),
       col=c(pal[3:1], "black"), lwd=3, pch=c(4, 18, 17, 15),
       xpd=NA, y.intersp=1.7, bty="n")


# Figure A6
cairo_pdf("Scenarios all countries.pdf")
par(mfrow=c(8, 3), mar=c(0.5, 3, 1, 0.1), cex=0.6)

pal <- RColorBrewer::brewer.pal(3, "Set1")

for(c in countries) {
    mods <- models[[c]]
    mnull <- mods$null
    mstable <- mods$stable
    mMF0 <- mods$MF0
    mMF <- mods$MF
    mMFT <- mods$MFT
    
    tab <- mnull$data
    
    years <- as.character(unique(tab$S))
    covnull <- sapply(years, cov.w, fitted(mnull))
    covstable <- sapply(years, cov.w, fitted(mstable))
    covMF0 <- sapply(years, cov.w, fitted(mMF0))
    covMF <- sapply(years, cov.w, fitted(mMF))
    covMFT <- sapply(years, cov.w, fitted(mMFT))
    covsat <- sapply(years, cov.w, tab$Freq)
    
    cvnull <- sqrt(covnull["Total:Total",])/covnull["Total",]
    cvstable <- sqrt(covstable["Total:Total",])/covstable["Total",]
    cvMF0 <- sqrt(covMF0["Total:Total",])/covMF0["Total",]
    cvMF <- sqrt(covMF["Total:Total",])/covMF["Total",]
    cvMFT <- sqrt(covMFT["Total:Total",])/covMFT["Total",]
    cvsat <- sqrt(covsat["Total:Total",])/covsat["Total",]
    
    plot(NULL, xlim=c(1973, 2016), ylim=c(.35, .8), type="b", main=countrynames[toupper(c)],
         yaxs="i", xlab="", ylab="", xaxt="n")
    axis(1, mgp=c(0, .5, 0), tcl=-.4, labels=c %in% tail(countries, 3))
    if(c %in% countries[seq(1, length(countries), by=3)])
        title(ylab="CV", line=2)
    grid(NA, NULL); box()
    #plot.val(cvnull, lwd=3, pch=1, col="grey")
    plot.val(cvstable, lwd=1.5, pch=4, col=pal[3])
    #plot.val(cvMF0, lwd=3, pch=19, col=pal[2])
    plot.val(cvMF, lwd=1.5, pch=18, col=pal[2])
    plot.val(cvMFT, lwd=1.5, pch=17, col=pal[1])
    plot.val(cvsat, lwd=1.5, pch=15)
}
plot.new()
plot.new()
par(mar=c(0, 0, 0, 0))
legend("center", c("S1: only earnings distributions change",
                   "S2: + change in employment rates",
                   "S3: + change in employment-partner's earnings association",
                   "S4: + change in association among dual-earner couples"),
       title=expression(bold("Counterfactual scenarios:")),
       col=c(pal[3:1], "black"), lwd=1.5, pch=c(4, 18, 17, 15),
       xpd=NA, y.intersp=1.7, bty="n", ncol=2)
dev.off()
