library(gnm)

models <- list()
for(c in countries) {
  cat(c, "\n")

  tab <- tabs[,, substr(dimnames(tabs)[[3]], 1, 2) == c]
  names(dimnames(tab)) <- c("M", "F", "S")

  tab <- as.data.frame(as.table(tab))
 
  tab$Y <- factor(substr(tab$S, 3, 4), levels=unique(substr(tab$S, 3, 4)))
  tab$F0 <- tab$F == "Inactive"
  tab$M0 <- tab$M == "Inactive"
  tab$Fnum <- as.numeric(ifelse(tab$F == "Inactive", 0, substr(tab$F, 2, 4)))
  tab$Mnum <- as.numeric(ifelse(tab$M == "Inactive", 0, substr(tab$M, 2, 4)))

  tab$liF <- medians[cbind(as.character(tab$M), "F", as.character(tab$S))]
  tab$liM <- medians[cbind(as.character(tab$F), "M", as.character(tab$S))]

  # To check that constraints are correct, use coef(m)[!is.na(coef(m))]:
  # non-NA coefficients are those which are unconstrained
  m1num <- gnm(Freq ~ M*F + F0*(Mnum + I(Mnum^2)) + M0*(Fnum + I(Fnum^2)),
               data=subset(tab, Y == levels(Y)[1]), family=quasipoisson,
               constrain="^MQ.*:FInactive$|^MInactive:FQ", constrainTo=rep(0, 2*19))
  # Objective is to reproduce fitted table for first year rather than actually observed table
  # (this limits problems due to imprecise estimation of association)
  tab$Freq[tab$Y == levels(tab$Y)[1]] <- fitted(m1num)

  m1sat <- gnm(Freq ~ M*F, data=subset(tab, Y == levels(Y)[1]), family=quasipoisson)
  cons <- pickCoef(m1sat, "M|F", value=TRUE)
  print(summary(cons))
  m1indep <- gnm(Freq ~ M + F, data=subset(tab, Y == levels(Y)[1]), family=quasipoisson)
  pat <- sprintf("^(M|F)(%s)$", paste(levels(tab$M), collapse="|"))
  mnull <- gnm(Freq ~ M + F + Y, data=tab, family=quasipoisson, trace=TRUE,
               constrain=pat, constrainTo=pickCoef(m1indep, pat, value=TRUE))
  mstable <- gnm(Freq ~ M + F + Y + M:F, data=tab, family=quasipoisson, trace=TRUE,
                 constrain=names(cons), constrainTo=cons)
  mMF0 <- gnm(Freq ~ M + F + M0*Y + F0*Y + M:F, data=tab, family=quasipoisson, trace=TRUE,
              constrain=names(cons), constrainTo=cons)
  # Model equivalent to the above one in terms of inequalities, modulo quantile approximation
  # (but gives different fit statistics)
  mMF <- gnm(Freq ~ M*Y + F*Y + M:F, data=tab, family=quasipoisson, trace=TRUE,
             constrain=names(cons), constrainTo=cons)
  # With linear trend
  # mMFT <- gnm(Freq ~ M*Y + F*Y + M:F + T*(F0:M) + T*(M0:F), data=tab, family=quasipoisson, trace=TRUE,
  #             constrain=sprintf("^(%s|.*[^T]:(M|F)0TRUE)$", paste(names(cons), collapse="|")),
  #             constrainTo=c(cons, rep(0, 42)))
  mMFT <- gnm(Freq ~ M*Y + F*Y + M:F + Y*(F0:M) + Y*(M0:F), data=tab, family=quasipoisson, trace=TRUE,
              constrain=names(cons), constrainTo=cons)
  # For checking purposes: give the same models as above (as long as we constrain the
  # correct coefficients: be careful about their order!)
  # mMF2 <- gnm(Freq ~ M*Y + F*Y + M:F, data=tab, family=quasipoisson, trace=TRUE,
  #             constrain="^M.*:F.*", constrainTo=pickCoef(m1sat, "^M.*:F.*", value=TRUE))
  # mMFT2 <- gnm(Freq ~ M*Y + F*Y + M:F + T*(F0:M) + T*(M0:F), data=tab, family=quasipoisson, trace=TRUE,
  #              constrain="^M[^0].*:F[^0].*|.*[^T]:(M|F)0TRUE$",
  #              constrainTo=c(pickCoef(m1sat, "^M.*:F.*", value=TRUE), rep(0, 42)))
  # all.equal(fitted(mMF), fitted(mMF2), check.attributes = F)
  # all.equal(fitted(mMFT), fitted(mMFT2), check.attributes = F)

  # Check consistency of models:
  y1 <- tab$Y == levels(tab$Y)[1]
  # - they should all reproduce the row/column sums for first year
  for(m in list(mnull, mstable, mMF0, mMF, mMFT)) {
    stopifnot(all.equal(tapply(fitted(m)[y1], list(tab$M[y1]), sum),
                        tapply(tab$Freq[y1], list(tab$M[y1]), sum), tolerance=1e-4))
    stopifnot(all.equal(tapply(fitted(m)[y1], list(tab$F[y1]), sum),
                        tapply(tab$Freq[y1], list(tab$F[y1]), sum), tolerance=1e-4))
  }
  # - mnull and mstable should use first year row/column profiles for all years
  for(m in list(mnull, mstable)) {
      for(y in levels(tab$Y)) {
          stopifnot(all.equal(prop.table(tapply(fitted(m)[tab$Y==y], list(tab$M[tab$Y==y]), sum)),
                              prop.table(tapply(tab$Freq[y1], list(tab$M[tab$Y==y]), sum)), tolerance=1e-4))
          stopifnot(all.equal(prop.table(tapply(fitted(m)[tab$Y==y], list(tab$F[y1]), sum)),
                              prop.table(tapply(tab$Freq[y1], list(tab$F[y1]), sum)), tolerance=1e-4))
      }
  }
  # - mMF0, mMF and mMFT should reproduce observed number of inactive for all years
  for(m in list(mMF0, mMF, mMFT)) {
      stopifnot(all.equal(tapply(fitted(m), list(tab$M, tab$Y), sum)["Inactive",],
                          tapply(tab$Freq, list(tab$M, tab$Y), sum)["Inactive",], tolerance=1e-4))
      stopifnot(all.equal(tapply(fitted(m), list(tab$F, tab$Y), sum)["Inactive",],
                          tapply(tab$Freq, list(tab$F, tab$Y), sum)["Inactive",], tolerance=1e-4))
  }
  # - mMF and mMFT should reproduce observed row/column sums for all years
  for(m in list(mMF, mMFT)) {
      stopifnot(all.equal(tapply(fitted(m), list(tab$M, tab$Y), sum),
                          tapply(tab$Freq, list(tab$M, tab$Y), sum), tolerance=1e-4))
      stopifnot(all.equal(tapply(fitted(m), list(tab$F, tab$Y), sum),
                          tapply(tab$Freq, list(tab$F, tab$Y), sum), tolerance=1e-4))
  }
  # - mnull should have null association for all years
  for(y in levels(tab$Y)) {
      stopifnot(sum(abs(logmult:::lambda(tapply(fitted(mnull)[tab$Y==y], list(tab$M[tab$Y==y], tab$F[tab$Y==y]), sum)))) < 1e-4)
  }
  # - mstable should reproduce first year proportions for all years
  for(y in levels(tab$Y)) {
      stopifnot(all.equal(prop.table(tapply(fitted(mstable)[tab$Y==y], list(tab$M[tab$Y==y], tab$F[tab$Y==y]), sum)),
                          prop.table(tapply(tab$Freq[y1], list(tab$M[y1], tab$F[y1]), sum)), tolerance=1e-4))
  }
  # - mstable, mMF0 and mMFT should should reproduce first year associations for all years
  # (we cannot compare association direclty with observed data due to zeros, which are no reproduced exactly,
  #  so check against mstable whose proportions in first year have been checked above)
  for(y in levels(tab$Y)) {
      stopifnot(all.equal(logmult:::lambda(tapply(fitted(mstable)[tab$Y==y], list(tab$M[tab$Y==y], tab$F[tab$Y==y]), sum)),
                          logmult:::lambda(tapply(fitted(mstable)[y1], list(tab$M[y1], tab$F[y1]), sum)), tolerance=1e-4))
      stopifnot(all.equal(logmult:::lambda(tapply(fitted(mMF0)[tab$Y==y], list(tab$M[tab$Y==y], tab$F[tab$Y==y]), sum)),
                          logmult:::lambda(tapply(fitted(mstable)[y1], list(tab$M[y1], tab$F[y1]), sum)), tolerance=1e-4))
      stopifnot(all.equal(logmult:::lambda(tapply(fitted(mMF)[tab$Y==y], list(tab$M[tab$Y==y], tab$F[tab$Y==y]), sum)),
                          logmult:::lambda(tapply(fitted(mstable)[y1], list(tab$M[y1], tab$F[y1]), sum)), tolerance=1e-4))
  }
  # - mstable, mMF0, mMF and mMFT should reproduce first year associations for all years among dual-earners
  # (we cannot compare association direclty with observed data due to zeros, which are no reproduced exactly,
  #  so check against mstable whose proportions in first year have been checked above)
  for(y in levels(tab$Y)) {
      stopifnot(all.equal(logmult:::lambda(tapply(fitted(mstable)[tab$Y==y], list(tab$M[tab$Y==y], tab$F[tab$Y==y]), sum)[1:20, 1:20]),
                          logmult:::lambda(tapply(fitted(mstable)[y1], list(tab$M[y1], tab$F[y1]), sum)[1:20, 1:20]), tolerance=1e-4))
      stopifnot(all.equal(logmult:::lambda(tapply(fitted(mMF0)[tab$Y==y], list(tab$M[tab$Y==y], tab$F[tab$Y==y]), sum)[1:20, 1:20]),
                          logmult:::lambda(tapply(fitted(mstable)[y1], list(tab$M[y1], tab$F[y1]), sum)[1:20, 1:20]), tolerance=1e-4))
      stopifnot(all.equal(logmult:::lambda(tapply(fitted(mMF)[tab$Y==y], list(tab$M[tab$Y==y], tab$F[tab$Y==y]), sum)[1:20, 1:20]),
                          logmult:::lambda(tapply(fitted(mstable)[y1], list(tab$M[y1], tab$F[y1]), sum)[1:20, 1:20]), tolerance=1e-4))
      stopifnot(all.equal(logmult:::lambda(tapply(fitted(mMFT)[tab$Y==y], list(tab$M[tab$Y==y], tab$F[tab$Y==y]), sum)[1:20, 1:20]),
                          logmult:::lambda(tapply(fitted(mstable)[y1], list(tab$M[y1], tab$F[y1]), sum)[1:20, 1:20]), tolerance=1e-4))
  }
  # - mMF0 and mMF should have equal associations for all years
  for(y in levels(tab$Y)) {
      stopifnot(all.equal(logmult:::lambda(tapply(fitted(mMF0)[tab$Y==y], list(tab$M[tab$Y==y], tab$F[tab$Y==y]), sum)),
                          logmult:::lambda(tapply(fitted(mMF)[tab$Y==y], list(tab$M[tab$Y==y], tab$F[tab$Y==y]), sum)), tolerance=1e-4))
  }
  # - mstable, mMF0, mMF and mMFT should reproduce observed values for first year
  for(m in list(mstable, mMF0, mMF, mMFT)) {
      stopifnot(all.equal(fitted(m)[y1], tab$Freq[y1], tolerance=1e-4, check.attributes=FALSE))
  }
  # - mMFT should reproduce observed row and column profiles for the inactive for all years
  for(y in levels(tab$Y)) {
      stopifnot(all.equal(prop.table(tapply(fitted(mMFT)[tab$Y==y], list(tab$M[tab$Y==y]), sum))["Inactive"],
                          prop.table(tapply(tab$Freq[tab$Y==y], list(tab$M[tab$Y==y]), sum))["Inactive"],
                          tolerance=1e-4))
      stopifnot(all.equal(prop.table(tapply(fitted(mMFT)[tab$Y==y], list(tab$F[tab$Y==y]), sum))["Inactive"],
                          prop.table(tapply(tab$Freq[tab$Y==y], list(tab$F[tab$Y==y]), sum))["Inactive"],
                          tolerance=1e-4))
  }

  models[[c]] <- list(null=mnull, stable=mstable, MF0=mMF0, MF=mMF, MFT=mMFT)
}

save(models, file="models.RData")