load("models.RData")

cov.w <- function(y, w, y.mediansM=y, y.mediansF=y) {
    m <- medians[as.character(tab$M[tab$S==y]), "M", as.character(y.mediansM)]
    f <- medians[as.character(tab$F[tab$S==y]), "F", as.character(y.mediansF)]
    tot <- m + f
    w <- w[tab$S==y]
    tot.mean <- sum(tot * w)/sum(w)
    m.mean <- sum(m * w)/sum(w)
    f.mean <- sum(f * w)/sum(w)
    
    res <- cov.wt(cbind(Total=tot, M=m, F=f),
                  w, method="ML")
    cov <- c(res$cov)
    names(cov) <- outer(c("Total", "M", "F"), c("Total", "M", "F"), FUN=paste, sep=":")
    c(cov, "Total"=tot.mean, "M"=m.mean, "F"=f.mean)
}

vals <- data.frame()
for(c in names(models)) {
    print(c)
    mods <- models[[c]]
    mnull <- mods$null
    mstable <- mods$stable
    mMF0 <- mods$MF0
    mMF <- mods$MF
    mMFT <- mods$MFT
    
    tab <- mnull$data
    
    years <- as.character(unique(tab$S))
    covnull <- sapply(years, cov.w, fitted(mnull))
    covstable <- sapply(years, cov.w, fitted(mstable))
    covMF0 <- sapply(years, cov.w, fitted(mMF0))
    covMF <- sapply(years, cov.w, fitted(mMF))
    covMFT <- sapply(years, cov.w, fitted(mMFT))
    covsat <- sapply(years, cov.w, tab$Freq)
    
    cvnull <- sqrt(covnull["Total:Total",])/covnull["Total",]
    cvstable <- sqrt(covstable["Total:Total",])/covstable["Total",]
    cvMF0 <- sqrt(covMF0["Total:Total",])/covMF0["Total",]
    cvMF <- sqrt(covMF["Total:Total",])/covMF["Total",]
    cvMFT <- sqrt(covMFT["Total:Total",])/covMFT["Total",]
    cvsat <- sqrt(covsat["Total:Total",])/covsat["Total",]

    newvals <- cbind(cvsat, cvMFT, cvMF, cvstable, cvnull)
    vals <- rbind(vals, newvals)
}

library(dplyr)
colnames(vals) <- paste0("CV", 4:0)
vals$country <- substr(rownames(vals), 1, 2)

# Effects expressed in % of observed CV (figures in the text)
diffs <- vals %>% group_by(country) %>%
    mutate_all(.funs=funs(D = (.-CV4)/CV4 * 100)) %>%
    mutate(D4 = CV4-first(CV4)) %>%
    filter(row_number() == n())

diffs <- diffs[c("country", "D4", "CV4", "CV3", "CV3_D", "CV2", "CV2_D", "CV1", "CV1_D")]
diffs <- diffs[match(c("de", "lu", "ch", "us", "fr", "at", "nl",
                       "si", "es", "cz", "fi", "hu", "ie", "gr",
                       "se", "no", "ca", "dk", "uk", "au", "it", "sk"),
                     diffs$country),]

# Table 1 (rightmost columns)
diffs2 <- vals %>% group_by(country) %>%
    mutate_all(.funs=funs(D = (.-CV4))) %>%
    mutate(CV4_change = CV4-first(CV4)) %>%
    filter(row_number() == n())
diffs2 <- diffs2[c("country", "CV4", "CV4_change", "CV3", "CV3_D", "CV2", "CV2_D", "CV1", "CV1_D")]
diffs2 <- diffs2[match(c("de", "lu", "ch", "us", "fr", "at", "nl",
                         "si", "es", "cz", "fi", "hu", "ie", "gr",
                         "se", "no", "ca", "dk", "uk", "au", "it", "sk"),
                       diffs2$country),]
cbind(diffs2$country, round(subset(diffs2, select=-country), 2))
mutate(diffs2, CV3_D/CV4_change, CV2_D/CV4_change)

# Group averages
diffs2$group <- with(diffs2, case_when(country %in% c("de", "lu", "ch", "us") ~ "Pattern 1: de",
                                       country %in% c("fr", "at", "nl", "si", "es") ~ "Pattern 2: fr",
                                       country %in% c("cz", "fi", "hu") ~ "Pattern 3: cz",
                                       country %in% c("ie", "gr") ~ "Pattern 4: ie",
                                       country %in% c("se", "no", "ca", "dk", "uk") ~ "Pattern 5: se"),
                                       country %in% c("au", "it") ~ "Pattern 6: other")

diffs2 %>% group_by(group) %>% summarise_if(is.numeric, function(x) round(mean(x), 2))
